from .general.generalsection import GeneralSection as GeneralSection
from .general.materialsection import MaterialSection as MaterialSection
from .general.compositesection import CompositeSection as CompositeSection
from .general.thinwalledsection import ThinWalledSection as ThinWalledSection
from .general.cripplingsection import CripplingSection as CripplingSection
