lunit = 'mm' # Length unit used in pysectprop
funit = 'N' # Force unit used in pysectprop
sunit = 'MPa' # Stress unit used in pysectprop

angfrm = '.1f' # Angle format
l1frm = '.1f' # Length format
l2frm = '.1f' # Length**2 format
l3frm = '.0f' # Length**3 format
l4frm = '.0f' # Length**4 format
sfrm = '.1f' # Stress format
efrm = '.0f' # Elastic Modulus formaty

msmode = False # Output Margins of Safety
